@@ -51,7 +51,7 @@ my_mpi_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status)
 PLUG_API void YpPush(const char *filename);      /* filename will be copied */
 
 PLUG_API ybuiltin_t Y_mp_send, Y_mp_recv, Y_mp_probe, Y_mp_exec;
-PLUG_API ybuiltin_t Y_mpy_nfan;
+PLUG_API ybuiltin_t Y_mpy_nfan, Y_mp_dbstate;
 
 /* The interpreter uses its own communicator to avoid conflicts with
  * any compiled parallel packages.
@@ -143,6 +143,9 @@ static int mpy_aborted = 0, mpy_batch = 0;
   MPI_Abort
  */
 
+PLUG_API int yerror_flags;  /* see yinput.c */
+static int y0error_flags = 0;
+
 static int mpy_initdone = 0;
 static char *mpy_batscript = 0;
 
@@ -166,7 +169,8 @@ mpy_initialize(int *pargc, char **pargv[])
         || MPI_Errhandler_set(mpy_world, MPI_ERRORS_RETURN) != MPI_SUCCESS) {
       if (mpy_initdone) MPI_Finalize(); /* may be a mistake - no return? */
       mpy_world = MPI_COMM_NULL;
-      mpy_size = mpy_rank = mpy_nfan = 0;
+      mpy_size = mpy_rank = 0;
+      mpy_nfan = 1;  /* allows mp_exec to work even if no MPI */
       mpy_take_back_signals();
       /* y_error("MPI initialization sequence failed"); */
       /* at least some MPI platforms arrive here when started in serial */
@@ -182,6 +186,7 @@ mpy_initialize(int *pargc, char **pargv[])
       mpy_batscript = pargv[0][2];
 
     y_errhook = mpy_recover;
+    if (mpy_rank) yerror_flags |= 1;  /* mpy_recover can override this */
   }
 }
 
@@ -430,7 +435,8 @@ Y_mp_exec(int argc)
         return;
       }
     }
-    mpy_bcast(1);  /* make sure everybody has char command on stack */ 
+    if (mpy_size > 1)
+      mpy_bcast(1);  /* make sure everybody has char command on stack */ 
     /* invoke interpreted include,char_array */
     if (mpy_rank) {
       char *cmsg = (yarg_typeid(0)==Y_CHAR)? ygeta_c(0,0,0) : 0;
@@ -448,7 +454,7 @@ Y_mp_exec(int argc)
        *   immediate inside Y_include on rank 0, and it will be
        *   a parallel task with other message passing
        */
-      mpy_bcast(0);
+      if (mpy_size > 1) mpy_bcast(0);
       Y_include(2);
       mpy_exec0(0);
     }
@@ -458,9 +464,6 @@ Y_mp_exec(int argc)
   }
 }
 
-PLUG_API int yerror_flags;  /* see yinput.c */
-static int y0error_flags = 0;
-
 static long mpy_after0 = -1, mpy_after1 = -1;
 static long mpy_after2 = -1, mpy_after3 = -1;
 void
@@ -473,17 +476,19 @@ mpy_exec0(int on)
     mpy_after3 = yget_global("mpy_frank", 0);
   }
   mpy_parallel = on;
-  if (on) {  /* after_error = mpy_after_error */
-    y0error_flags = yerror_flags;
-    yerror_flags |= 1;
-    ypush_global(mpy_after1);
-    yput_global(mpy_after0, 0);
-    yarg_drop(1);
-  } else {   /* after_error = [] */
-    yerror_flags = y0error_flags;
-    ypush_nil();
-    yput_global(mpy_after0, 0);
-    yarg_drop(1);
+  if (mpy_size > 1) {
+    if (on) {  /* after_error = mpy_after_error */
+      y0error_flags = yerror_flags;
+      yerror_flags |= 1;
+      ypush_global(mpy_after1);
+      yput_global(mpy_after0, 0);
+      yarg_drop(1);
+    } else {   /* after_error = [] */
+      yerror_flags = y0error_flags;
+      ypush_nil();
+      yput_global(mpy_after0, 0);
+      yarg_drop(1);
+    }
   }
 }
 
@@ -1197,12 +1202,16 @@ static void
 mperr_fatal(const char *msg)
 {
   /* do not be clever here -- just kill all processes */
-  if (!mpy_batch)
-    y_warnn("Rank %ld raised fatal MPI communication error:", mpy_rank);
-  else
-    y_warnn("Rank %ld raised error in batch mode:", mpy_rank);
-  y_warn(msg);
+  if (mpy_rank <= 16) {
+    if (!mpy_batch)
+      y_warnn("Rank %ld raised fatal MPI communication error:", mpy_rank);
+    else
+      y_warnn("Rank %ld raised error in batch mode:", mpy_rank);
+    y_warn(msg);
+  }
   mpy_abort();
+  if (!mpy_rank)
+    y_warn("MPI crashed and aborted: only rank<=16 allowed to print errors");
 }
 
 /* ------------------------------------------------------------------------ */
