@@ -8,6 +8,7 @@
  * Read the accompanying LICENSE file for details.
  */
 
+#include "yapi.h"
 #include "ydata.h"
 #include "yio.h"
 #include "pstdlib.h"
@@ -37,6 +38,8 @@ extern BuiltIn Y_current_include, Y_get_includes;
 extern BuiltIn Y_plug_in, Y_plug_dir, Y_maybe_prompt, Y_suspend, Y_resume;
 extern BuiltIn Y_after, Y__after_func, Y_include1, Y_vopen, Y_vclose;
 
+extern ybuiltin_t Y_prompt_marker;
+
 extern void YRun(void);
 extern void YHalt(void);
 
@@ -123,6 +126,8 @@ YRun(void)
   ym_state |= Y_RUNNING;
   ym_dbenter = 0;
 
+  P_SOFTFPE_TEST;
+
   while (!p_signalling) {
     Action = (pc++)->Action;
     Action();
@@ -133,6 +138,8 @@ YRun(void)
   /* reset Y_RUNNING to value on entry -- allows YRun to recurse */
   ym_state = (ym_state & ~Y_RUNNING) | run_state;
 
+  P_SOFTFPE_TEST;
+
   if (p_signalling)
     p_abort();             /* p_signalling set by real signal */
 }
@@ -288,6 +295,23 @@ int yBatchMode= 0;
 
 static int y_was_idle= 0;
 
+/* prompt marker after prompts simplifies writing yorick controllers */
+static char *prompt_marker = 0;
+void
+Y_prompt_marker(int argc)
+{
+  char *marker = (argc==1)? ygets_q(0) : 0;
+  if (argc > 1) y_error("prompt_marker expecting single string argument");
+  if (marker && !marker[0]) marker = 0;
+  if (prompt_marker) {
+    char *old = prompt_marker;
+    prompt_marker = 0;
+    p_free(old);
+  }
+  if (marker)
+    prompt_marker = p_strcpy(marker);
+}
+
 static void ym_prompter(void);
 extern char *y_read_prompt;     /* ascio.c */
 /* yp_did_prompt reset by y_on_stdin */
@@ -301,12 +325,14 @@ ym_prompter(void)
     if (y_read_prompt) {
       if (y_read_prompt[0]) {
         p_stdout(y_read_prompt);
+        if (prompt_marker) p_stdout(prompt_marker);
         yp_did_prompt = 1;
       }
     } else {
       extern void y_do_prompt(void);  /* yorick.c */
       if (!yg_blocking) {
         y_do_prompt();
+        if (prompt_marker) p_stdout(prompt_marker);
         yp_did_prompt = 1;
       }
     }
@@ -564,6 +590,8 @@ ym_after_dq(int i)
   }
 }
 
+static int startup_done = 0;
+
 int
 y_on_idle(void)
 {
@@ -579,6 +607,8 @@ y_on_idle(void)
     taskCodeInit = 1;
   }
 
+  p_fpehandling(2);  /* be sure yorick FPE handling set properly */
+
   if (ym_state & Y_QUITTING) {
   die_now:
     y_cleanup();
@@ -590,7 +620,17 @@ y_on_idle(void)
     if (nYpIncludes || nYpInputs) {
       YpParse((void *)0);
     } else {
-      pending_stdin = y_pending_stdin();
+      if (startup_done) {
+        pending_stdin = y_pending_stdin();
+      } else {
+        /* make sure idler gets called once before any stdin
+         * - without this, starting with shell here-document makes
+         *   lines passed as here-document execute before custom.i,
+         *   not what user expects
+         */
+        if (!y_idler_function) pending_stdin = y_pending_stdin();
+        startup_done = 1;
+      }
       if (!(nTasks+caughtTask || nYpIncludes || nYpInputs) &&
           y_idler_function) {
         Function *f = y_idler_function;
@@ -767,6 +807,28 @@ yexec_include(int iarg, int now)
   Y_include(2);
 }
 
+void
+ytask_push(int iarg)
+{
+  if (iarg>=0 && yarg_func(iarg)==1) {
+    Function *f = (Function *)sp[-iarg].value.db;
+    PushTask(Ref(f));
+  } else {
+    YError("can only run interpreted functions as tasks");
+  }
+}
+
+void
+ytask_run(int iarg)
+{
+  if (iarg>=0 && yarg_func(iarg)==1) {
+    Function *f = (Function *)sp[-iarg].value.db;
+    RunTaskNow(f);
+  } else {
+    YError("can only push interpreted functions onto task stack");
+  }
+}
+
 void
 Y_include1(int nArgs)
 {
@@ -855,7 +917,7 @@ typedef struct y_vopen_t y_vopen_t;
 struct y_vopen_t {
   p_file_ops *ops;
   Array *array;
-  long addr, maxaddr;
+  long addr, maxaddr, offset;
   int binary;
 };
 
@@ -865,7 +927,7 @@ ynew_vopen(Array *array, int binary)
   y_vopen_t *file = p_malloc(sizeof(y_vopen_t));
   file->ops = &y_vopen_ops;
   file->array = Ref(array);
-  file->addr = 0;
+  file->addr = file->offset = 0;
   file->maxaddr = (binary&2)? 0 : ((y_vopen_t *)file)->array->type.number;
   file->binary = binary;
   return (p_file *)file;
@@ -1005,6 +1067,7 @@ yv_fseek(p_file *file, unsigned long addr)
   }
   if (addr<0 || addr>len) return -1;
   ((y_vopen_t *)file)->addr = addr;
+  ((y_vopen_t *)file)->offset = 0;
   if (addr > ((y_vopen_t *)file)->maxaddr)
     ((y_vopen_t *)file)->maxaddr = addr;
   return 0;
@@ -1022,6 +1085,7 @@ yv_fgets(p_file *file, char *buf, int buflen)
     jeof = ((y_vopen_t *)file)->array->type.number - ((y_vopen_t *)file)->addr;
   } else {
     txt = ((y_vopen_t *)file)->array->value.q[((y_vopen_t *)file)->addr];
+    txt += ((y_vopen_t *)file)->offset;
     jeof = 0L;
   }
   if (buflen <= 0) return 0;
@@ -1038,8 +1102,15 @@ yv_fgets(p_file *file, char *buf, int buflen)
     buf[i] = c;
   }
   buf[i] = '\0';
-  if (!strng) ((y_vopen_t *)file)->addr += j;
-  else ((y_vopen_t *)file)->addr++;
+  if (!strng) {
+    ((y_vopen_t *)file)->addr += j;
+  } else if (i>=buflen-1 && txt && txt[j] && j>0) {
+    /* handle case where fgets stopped because buflen too short to hold line */
+    ((y_vopen_t *)file)->offset += j;
+  } else {
+    ((y_vopen_t *)file)->addr++;
+    ((y_vopen_t *)file)->offset = 0;
+  }
   return buf;
 }
 
@@ -1465,8 +1536,9 @@ Function *FuncContaining(Instruction *pc)
   Function *func= 0;
 
   if (!findingFunc && pc) {
-    long i= -1;
-    findingFunc= 1;
+    long i = -1;
+    if (pc>=taskCode && pc<=taskCode+4) return 0;
+    findingFunc = 1;
     for (;; i++) {
       while (pc[i].Action) i++;
       if (pc[i-1].Action==&Return) break;
@@ -1477,11 +1549,11 @@ Function *FuncContaining(Instruction *pc)
           vmCode[nextPC].index= codeSize= nPos+nKey+nLocal+ nextPC;
        (nextPC does NOT include the parameters or locals)
      */
-    i-= pc[i].index;
+    i -= pc[i].index;
     if (i<0) {
       /* see also Pointee function in ydata.c */
-      func= (Function *)((char *)(pc+i) - offsetof(Function, code));
-      findingFunc= 0;
+      func = (Function *)((char *)(pc+i) - offsetof(Function, code));
+      findingFunc = 0;
     }
   }
 
@@ -1489,7 +1561,7 @@ Function *FuncContaining(Instruction *pc)
     /* may get here after a disaster causing an interrupt above, as well
        as after scanning from a garbled initial pc */
     int no_pf = ((yerror_flags&1) != 0);
-    findingFunc= 0;
+    findingFunc = 0;
     if (!no_pf) YputsErr("(BUG) lost function produced following error:");
   }
   return func;
@@ -1648,11 +1720,10 @@ YError(const char *msg)
     char *tmp= includeFile;
     includeFile= 0;
     p_free(tmp);
-    if (nYpIncludes) {
+    if (nYpIncludes)
       includeFile= p_strcpy(ypIncludes[nYpIncludes-1].filename);
-      YpClearIncludes();
-    }
-  } else if (nYpIncludes) YpClearIncludes();
+  }
+  YpClearIncludes();
 
   /* Clean up any Array temporaries (used for data format conversions).  */
   if (recursing<2) ClearTmpArray();
@@ -1910,7 +1981,7 @@ void Y_error(int nArgs)
   else YError("<interpreted error function called>");
 }
 
-/* FIXME: this shold also turn off on_stdin event handling */
+/* FIXME: this should also turn off on_stdin event handling */
 void Y_batch(int nArgs)
 {
   int flag= 2;
