@@ -1030,7 +1030,7 @@ ypush_ptr(ypointer_t ptr, long *ntot)
 {
   int typeid = Y_VOID;
   Array *a = Pointee(ptr);
-  PushDataBlock(a);
+  PushDataBlock(Ref(a));
   if (a && (a != (Array*)&nilDB)) {
     yget_dims(ntot, (long*)0, &a->type);
     typeid = a->ops->typeID;
@@ -1361,9 +1361,14 @@ y_uo_extract(Operand *op, char *name)
   if (uo->uo_type->uo_ops != uo->ops)
     y_error("(BUG) corrupted user object in y_uo_extract");
   if (uo->uo_type->on_extract) {
-    long owner = op->owner - spBottom;
+    Symbol *stack;
+    long owner = op->owner - spBottom, sp0 = sp - spBottom;
     uo->uo_type->on_extract(uo->body.c, name);
     PopTo(spBottom+owner);
+    while (sp - spBottom > sp0) {
+      stack = sp--; /* sp decremented BEFORE stack element is deleted */
+      if (stack->ops == &dataBlockSym) Unref(stack->value.db);
+    }
   } else {
     y_error("user object has no on_extract method");
   }
